use chrono::Local;
use dataworker::dtos::Root;
use dataworker::insert_document;
use sqlx::postgres::PgPoolOptions;

use dotenvy::dotenv;
use std::env;
use std::io::Write;
use tokio::time::{self, Duration};

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    dotenv().ok();
    let stations: Vec<_> = env::var("STATIONS")?
        .split(',')
        .map(|s| s.to_string())
        .collect();
    let token = env::var("VRN_TOKEN")?;
    let req = env::var("REQ")?;
    let url = env::var("DATABASE_URL")?;
    let mut conn = PgPoolOptions::new()
        .max_connections(5)
        .connect(&url)
        .await?;

    let total = stations.len();
    let mut count = 0;

    let client = reqwest::Client::new();
    let mut interval = time::interval(Duration::from_secs(1800));

    loop {
        print!("\nWaiting for next interval tick... ");
        std::io::stdout().flush().unwrap();
        interval.tick().await;
        let date = Local::now();
        println!("Ticked on {}", &date.format("%Y.%m.%d %H:%M"));
        for station in &stations {
            count += 1;
            let req = req
                .replace("TEMPLATE_ID", &station)
                .replace("TEMPLATE_DATE", &date.format("%Y%m%d").to_string())
                .replace("TEMPLATE_TIME", &date.format("%H%M").to_string());
            print!("\x1B[2K\rFetching {}/{} ({}) ", count, total, station);
            std::io::stdout().flush().unwrap();
            if let Ok(doc) = client
                .get(req.replace("TEMPLATE_DEPARR", "arr"))
                .header("Authorization", &token)
                .send()
                .await?
                .json::<Root>()
                .await
            {
                match insert_document(doc, &mut conn).await {
                    Ok(_) => {}
                    Err(e) => println!("\x1B[2K\rError inserting document: {}", e),
                }
            } else {
                println!("\x1B[2K\rFailed to fetch arrivals for station {}", station);
            }
            if let Ok(doc) = client
                .get(req.replace("TEMPLATE_DEPARR", "dep"))
                .header("Authorization", &token)
                .send()
                .await?
                .json::<Root>()
                .await
            {
                match insert_document(doc, &mut conn).await {
                    Ok(_) => {}
                    Err(e) => println!("\x1B[2K\rError inserting document: {}", e),
                }
            } else {
                println!("\x1B[2K\rFailed to fetch departures for station {}", station);
            }
        }
        count = 0;
    }
}
