use serde::{Deserialize, Serialize};
use serde_json::Value;
use std::collections::HashMap;

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Root {
    pub parameters: Vec<Parameter>,
    pub dm: Dm,
    pub arr: Arr,
    pub date_time: DateTime,
    pub date_range: Vec<DateRange>,
    pub serving_lines: ServingLines,
    pub departure_list: Option<List<DepartureList>>,
    pub arrival_list: Option<Vec<ArrivalList>>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Parameter {
    pub name: String,
    pub value: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Dm {
    pub input: Input,
    pub points: Points,
    pub itd_odv_assigned_stops: ItdOdvAssignedStops,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Input {
    pub input: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Points {
    pub point: Point,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Point {
    pub usage: String,
    #[serde(rename = "type")]
    pub type_field: String,
    pub name: String,
    pub stateless: String,
    pub any_type: String,
    pub sort: String,
    pub quality: String,
    pub best: String,
    pub object: String,
    #[serde(rename = "ref")]
    pub ref_field: Ref,
    pub infos: Option<Infos>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Ref {
    pub id: String,
    pub gid: String,
    pub omc: String,
    #[serde(rename = "placeID")]
    pub place_id: String,
    pub place: String,
    pub coords: Option<String>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Infos {
    pub info: Info,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Info {
    pub info_link_text: String,
    #[serde(rename = "infoLinkURL")]
    pub info_link_url: String,
    pub info_text: InfoText,
    pub param_list: Vec<ParamList>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct InfoText {
    pub content: String,
    pub subtitle: String,
    pub subject: String,
    pub additional_text: String,
    pub html_text: String,
    pub wml_text: String,
    pub sms_text: String,
    pub speech_text: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ParamList {
    #[serde(rename = "type")]
    pub type_field: String,
    pub name: String,
    pub value: String,
    pub edit: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ItdOdvAssignedStops {
    #[serde(rename = "stopID")]
    pub stop_id: String,
    pub name: String,
    pub x: Option<String>,
    pub y: Option<String>,
    pub map_name: Option<String>,
    pub value: String,
    pub place: String,
    pub name_with_place: String,
    pub distance_time: String,
    pub is_transfer_stop: String,
    pub vm: String,
    pub gid: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Arr {
    pub input: Input2,
    pub points: Value,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Input2 {
    pub input: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct DateTime {
    pub deparr: String,
    pub ttp_from: String,
    pub ttp_to: String,
    pub year: String,
    pub month: String,
    pub day: String,
    pub hour: String,
    pub minute: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct DateRange {
    pub day: String,
    pub month: String,
    pub year: String,
    pub weekday: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct PtOption {
    pub active: String,
    pub max_changes: String,
    pub max_time: String,
    pub max_wait: String,
    pub route_type: String,
    pub change_speed: String,
    pub line_restriction: String,
    pub use_prox_foot_search: String,
    pub use_prox_foot_search_origin: String,
    pub use_prox_foot_search_destination: String,
    pub bike: String,
    pub plane: String,
    pub no_crowded: String,
    pub no_solid_stairs: String,
    pub no_escalators: String,
    pub no_elevators: String,
    pub low_platform_vhcl: String,
    pub wheelchair: String,
    pub need_elevated_plt: String,
    pub assistance: String,
    #[serde(rename = "SOSAvail")]
    pub sosavail: String,
    pub no_lonely_transfer: String,
    pub illum_transfer: String,
    pub overground_transfer: String,
    pub no_insecure_places: String,
    pub private_transport: String,
    pub excluded_means: Vec<ExcludedMean>,
    pub active_imp: String,
    pub active_com: String,
    pub active_sec: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ExcludedMean {
    pub means: String,
    pub value: String,
    pub selected: String,
}

#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Single<T> {
    #[serde(flatten)]
    pub inner: HashMap<String, T>,
}

#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(untagged)]
pub enum List<T> {
    None,
    Single(Single<T>),
    Multiple(Vec<T>),
}

impl<T> IntoIterator for List<T> {
    type Item = T;
    type IntoIter = std::vec::IntoIter<Self::Item>;
    fn into_iter(self) -> Self::IntoIter {
        match self {
            Self::None => vec![].into_iter(),
            Self::Single(line) => line.inner.into_values().collect::<Vec<_>>().into_iter(),
            Self::Multiple(lines) => lines.into_iter(),
        }
    }
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ServingLines {
    pub lines: Option<List<Option<Line>>>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Line {
    pub mode: Mode,
    pub index: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Mode {
    pub name: String,
    pub number: String,
    pub product: String,
    pub product_id: String,
    #[serde(rename = "type")]
    pub type_field: String,
    pub code: String,
    pub destination: String,
    #[serde(rename = "destID")]
    pub dest_id: String,
    pub desc: String,
    pub timetable_period: String,
    pub diva: Diva,
    pub train_type: Option<String>,
    pub train_name: Option<String>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Diva {
    pub branch: String,
    pub line: String,
    pub supplement: String,
    pub dir: String,
    pub project: String,
    pub network: String,
    pub stateless: String,
    pub trip_code: String,
    pub operator: String,
    pub op_code: String,
    pub v_f: String,
    pub v_to: String,
    pub line_display: String,
    pub attrs: Vec<Attr>,
    pub op_public_code: Option<String>,
    pub global_id: Option<String>,
    pub special_fares: Option<String>,
    #[serde(rename = "isTTB")]
    pub is_ttb: Option<String>,
    #[serde(rename = "isSTT")]
    pub is_stt: Option<String>,
    #[serde(rename = "isROP")]
    pub is_rop: Option<String>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Attr {
    pub name: String,
    pub value: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct DepartureList {
    #[serde(rename = "stopID")]
    pub stop_id: String,
    pub x: Option<String>,
    pub y: Option<String>,
    pub map_name: Option<String>,
    pub area: String,
    pub platform: String,
    pub platform_name: String,
    pub stop_name: String,
    #[serde(rename = "nameWO")]
    pub name_wo: String,
    pub point_type: Option<String>,
    pub countdown: String,
    pub date_time: DateTime2,
    pub real_date_time: Option<DateTime2>,
    pub serving_line: ServingLine,
    pub operator: Option<Operator>,
    pub attrs: Option<Vec<Attr>>,
    pub stop_infos: Option<StopInfos>,
    pub line_infos: Option<Value>,
    pub trip_infos: Option<Value>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ArrivalList {
    #[serde(rename = "stopID")]
    pub stop_id: String,
    pub x: String,
    pub y: String,
    pub map_name: Option<String>,
    pub area: String,
    pub platform: String,
    pub platform_name: String,
    pub stop_name: String,
    #[serde(rename = "nameWO")]
    pub name_wo: String,
    pub countdown: String,
    pub date_time: DateTime2,
    pub real_date_time: Option<DateTime2>,
    pub serving_line: ArrServingLine,
    pub operator: Option<Operator>,
    pub attrs: Option<Vec<Attr>>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ArrServingLine {
    pub key: String,
    pub code: String,
    pub number: String,
    pub symbol: String,
    pub mot_type: String,
    pub mt_subcode: String,
    pub realtime: String,
    pub direction: String,
    pub direction_from: String,
    pub train_type: Option<String>,
    pub train_name: Option<String>,
    pub train_num: Option<String>,
    pub name: String,
    pub delay: Option<String>,
    pub hints: Option<Vec<Hint>>,
    pub li_erg_ri_proj: ArrLiErgRiProj,
    #[serde(rename = "destID")]
    pub dest_id: String,
    pub stateless: String,
    pub line_display: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct LiErgRiProj {
    pub line: String,
    pub project: String,
    pub direction: String,
    pub supplement: String,
    pub network: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct DateTime2 {
    pub year: String,
    pub month: String,
    pub day: String,
    pub weekday: String,
    pub hour: String,
    pub minute: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ServingLine {
    pub key: String,
    pub code: String,
    pub number: String,
    pub symbol: String,
    pub mot_type: String,
    pub mt_subcode: String,
    pub realtime: String,
    pub direction: String,
    pub direction_from: String,
    pub name: String,
    pub li_erg_ri_proj: LiErgRiProj,
    #[serde(rename = "destID")]
    pub dest_id: String,
    pub stateless: String,
    pub line_display: String,
    pub train_type: Option<String>,
    pub train_name: Option<String>,
    pub train_num: Option<String>,
    #[serde(default)]
    pub hints: Vec<Hint>,
    pub delay: Option<String>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ArrLiErgRiProj {
    pub line: String,
    pub project: String,
    pub direction: String,
    pub supplement: String,
    pub network: String,
    pub gid: Option<String>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Hint {
    pub content: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Operator {
    pub code: String,
    pub name: String,
    pub public_code: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct StopInfos {
    pub stop_info: StopInfo,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct StopInfo {
    pub info_link_text: String,
    #[serde(rename = "infoLinkURL")]
    pub info_link_url: String,
    pub info_text: InfoText2,
    pub param_list: Vec<ParamList2>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct InfoText2 {
    pub content: String,
    pub subtitle: String,
    pub subject: String,
    pub additional_text: String,
    pub html_text: String,
    pub wml_text: String,
    pub sms_text: String,
    pub speech_text: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ParamList2 {
    #[serde(rename = "type")]
    pub type_field: String,
    pub name: String,
    pub value: String,
    pub edit: String,
}
